# encoding : utf-8
# frozen_string_literal: true
module TTWatcher
  module Sites # no-doc
    extend self

    ##
    # Returns all possible pairs:
    #   :name => TorrentSite child class (with associated +:name+)
    #
    # @example
    #   torrent_sites[:megashara] # ==> Sites::Megashara
    #
    # @return [Hash<Symbol:Class>]

    def torrent_sites
      @known_torrent_sites ||=
        begin
          sites =
            Sites.constants.select do |const_name|
              const = Sites.const_get const_name
              const.class == Class && const < TorrentSite ? const : nil
            end # ==> [Array<Symbols>]

          hsh = Hash.new
          sites.each { |site| hsh[site.downcase] = Sites.const_get(site) }

          hsh
        end
    end

    ##
    # Returns a TorrentSite child class.
    #
    # @param [String, Symbol] name
    #   Site name.
    #
    # @exception TorrentSiteUnknown
    #   Raised when TorrentSite child class associated with +name+ not found.
    #
    # @return [TorrentSite]

    def fetch_torrent_site(name)
      site = torrent_sites[name.downcase.to_sym]

      if site.nil?
        raise TorrentSiteUnknown, name
      else
        return site
      end
    end

    ##
    # Returns list of known torrent sites.
    #
    # @return [Array<Symbols>]

    def list
      @torrent_sites_keys = torrent_sites.keys
    end

    ##
    # Raised when TorrentSite child class not found by it's name.

    class TorrentSiteUnknown < TTWError
      def initialize(name); super "Program does not know how to work with #{name} site"; end; end
  end # module Sites
end # module TTWatcher
