# encoding: utf-8
# frozen_string_literal: true
module TTWatcher
module Sites
  class Connection # no-doc
    include InternetConnection

    # @return [Site]
    #   Site that uses current connection.

    attr_reader :site

    #  @return [InternetConnection::Url]
    #    Last used url.

    attr_reader :url

    # @return [Hash]
    #   Default settings for the connection.

    def settings
      @settings ||=
        begin
          defaults = S[:defaults][:connection_settings].dup || {}
          defaults.deep_merge! S[site.name][:connection_settings]
          defaults
        end
    end

    ##
    # Creates new Connection instance.
    #
    # @return [Sites::Connection]

    def initialize(site)
      @site = site
    end

    ##
    # Downloads requested url.
    #
    # @param [String] url
    #   Url in any form: "https://site.com", "site.com", "www.site.com"
    #
    # @param [Hash] request_params
    #
    # @return [String, NilClass]
    #   when 'OK' it returns requested page (html, string).
    #   when 'Error' it returns +nil+.

    def download(url, request_params = {})
      with_exceptions_control do
        request = settings.dup.deep_merge request_params
        @url = Url.new url, request[:url]
        response = RestClient::Request.execute method: :get, url: url.to_s

        return response.body
      end
    end

    private

    def with_exceptions_control # no-doc
      yield

    rescue Errno::ECONNREFUSED # ==> Scheme not supported. Try another one.
      url.scheme_switch

      retry
    rescue RestClient::Forbidden
      Logger << "Connection for '#{url.to_s}' Forbidden."

      return nil
    rescue Exception => exception
      Logger.with_backtrace "Unexpected exception (#{exception.inspect}) " \
                            "has been raised for '#{url.to_s}' url."

      return nil
    end
  end # class Connection
end # module Sites
end # module TTWatcher
