# encoding: utf-8
# frozen_string_literal: true
module TTWatcher
module Parsers
  class SimpleParser < Parsers::Base # no-doc

    # @param [String] page
    #   Url to initial page for parsing.
    #
    # @return [TorrentList, nil]
    #   When any exception raised it returns nil
    #   Otherwise it returns TorrentList instance.

    def parse(page)
      return nil if page.is_a? NilClass
      self.page = page
      torrents = TorrentList.new
      loop do
        torrents << extract_torrents_from_page
        next_page? ? continue_with_next_page : break
      end

      return torrents
    rescue Exception => exception
      notificate_about_crash! exception

      return nil
    end

    private

    # @return Array<String>
    #   List of urls that should be scanned before complete torrents search.

    def new_pages_list
      raise NotImplementedError, "Abstract method called!"
    end

    # @return Array<Nokogiri::Node>
    #   Each element from this array represents all available information
    #   about 1 torrent.

    def torrents_unparsed
      raise NotImplementedError, "Abstract method called!"
    end

    ##
    # Extracts single torrent from +unparsed_data+.
    #
    # @param [Nokogiri::Node] unparsed_data
    #
    # @return [Torrent]

    def extract_torrent(unparsed_data)
      raise NotImplementedError, "Abstract method called!"
    end

    # @return [Encoding] (Encoding::UTF_8)
    #   Parser encoding preferences.

    attr_reader :encoding

    # @param [Encoding, String] new_encoding

    def encoding=(new_encoding)
      @encoding =
      if new_encoding.is_a?(Encoding)
        new_encoding
      else
        Encoding.find new_encoding
      end
    end

    # @return [String]
    #   Current page.

    attr_reader :page

    def page=(other_page)
      @page =
        if other_page.encoding.name == encoding.name
          other_page
        else
          other_page.force_encoding encoding
        end
    end

    ##
    # Structure for current +page+.
    #
    # @return [Nokogiri::HTML::Document]

    def structure
      Nokogiri::HTML page, nil, encoding.to_s
    end

    # @return [TorrentList]
    #  Returns extracted torrents from current +page+.

    def extract_torrents_from_page
      list = TorrentList.new
      torrents_unparsed.each do |unparsed_torrent|
        torrent = extract_torrent unparsed_torrent
        list << torrent
      end

      return list
    end

    # @return [TrueClass, FalseClass]
    #   Returns +true+ if some pages wasn't scanned, otherwise returns +false+.

    def next_page?
      new_pages_list.count > 0
    end

    def continue_with_next_page # no-doc
      url = new_pages_list.pop
      self.page = assigned_site.download_page url
    end

    def initialize(site) # no-doc
      super

      self.encoding =
        settings[:encoding] ? settings[:encoding] : Encoding::UTF_8
    end

    def notificate_about_crash!(exception)
      Logger.with_backtrace "Parser #{self.class} crashed with error:\n #{exception.inspect}"
    end
  end # class SimpleParser
end # module Parsers
end # module TTWatcher
