# encoding: utf-8
# frozen_string_literal: true
module TTWatcher
module Sites
  class Site

    # @return [Symbol]
    #   Site name (autogenerated, depends from class name).

    attr_reader :name

    # @return [TTWatcher::Connection]

    attr_reader :connection

    # @return [String]
    #   Returns full domain name.

    def domain_name
      @domain_name ||=
        if S[name][:domain_name]
          S[name][:domain_name]
        else
          raise DomainNameNotFound, self
        end
    end

    ##
    # Creates new Site object.
    #
    # @return [Site]

    def initialize
      @name = H::class_name self
      @connection = Connection.new self
    end

    ##
    # Generates an url that includes scheme + domain name + +path+ (optional)
    #
    # example: input  ==> @domain_name = "some.site.com", path = 'hello/world'
    #          output ==> "http://some.site.com/hello/world"
    #
    # if path not selected it returns scheme + +@domain_name+

    def address(path='')
      scheme = connection.url.scheme
      if !path.empty? && domain_name_included?(path)
        InternetConnection::Scheme.add_scheme!(path, scheme)
      else
        hn = domain_name.dup # prevent unnecessary +domain_name+ mutation
        InternetConnection::Scheme.add_scheme!(hn, scheme)
        path = '/' + path unless path[0] == '/'
        hn + path
      end
    end

    # @param [String] url
    #
    # @param [Hash] request_params
    #
    # @return [String, NilClass]

    def download_page(url, request_params = {})
      url = address(url) unless domain_name_included? url
      page = connection.download url, request_params

      return page
    end

    private

    # @param [String] url
    #   Url or an url part.
    #
    # @return [TrueClass, FalseClass]
    #  +true+ when url includes domain name
    #  +false+ otherwise.

    def domain_name_included?(url)
      url[domain_name.to_s]
    end

    ##
    # Raised when param +domain name+ not found.

    class DomainNameNotFound < TTWError
      def initialize(klass) "Param ('domain_name') not found for #{klass.class} in config.yml." end
    end
  end # class Site
end # module Sites
end # module TTWatcher
